/* -------------------------------------------------------------------------
 | Copyright (C) 2006  Ali Rawashdeh
 | This program is free software; you can redistribute it and/or modify
 | it under the terms of the GNU General Public License as published by
 | the Free Software Foundation; either version 2 of the License, or
 | (at your option) any later version.
 | 
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 | GNU General Public License for more details.
 | 
 | You should have received a copy of the GNU General Public License along
 | with this program (see License.txt); if not, write to the Free Software
 | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/

using System;
using HomeDev.RSSTicker.Objects;
using HomeDev.Useful;
using System.Timers;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using System.Threading;

namespace HomeDev.RSSTicker.Controller
{
		public delegate void RestartTickThreadDelegate();

	/// <summary>
	/// This class controls the flow of execution of the program
	/// </summary>
	public class Controller
	{
		// Declarations
		private System.Timers.Timer feedsTimer_i;
		private Bookmarks bookmarks_i;
		private Settings settings_i;
		private Panel panelToUpdate_i; 
		private ArrayList displayedLabels_i;
		private int currentLabelPos_i;
		private ArrayList currentSetOfLabels_i;
		private Feeds nextFeeds_i;
		private int currentScrollSpeed_i;
		private RestartTickThreadDelegate restartTickThreadDelegate_i;

		#region ctor

		public Controller(Panel panelToUpdate_p, RestartTickThreadDelegate restartTickThreadDelegate_p)
		{
			// Assign the panel to update and initialise the label array
			displayedLabels_i = new ArrayList();
			panelToUpdate_i = panelToUpdate_p;
			currentLabelPos_i = 0;

			// Assign the thread restart delegate so that the ticker can start when the first bookmark is added
			restartTickThreadDelegate_i = restartTickThreadDelegate_p;

			// Load the bookmarks and settings from the xml files
			this.bookmarks_i = LoadBookmarks();
			this.settings_i = LoadSettings();
			this.currentScrollSpeed_i = GetScrollSpeedFromSetting(settings_i.Speed);

			// Update the next feeds object based on the bookmarks loaded
			nextFeeds_i = new Feeds(this.bookmarks_i);
			nextFeeds_i.UpdateFeeds();
			this.currentSetOfLabels_i = this.GetNewSetOfLabels();
			
			// Initialise the feeds timer to update the feeds
			feedsTimer_i = new System.Timers.Timer();
			feedsTimer_i.Interval = this.settings_i.UpdatePeriod * 1000;
			feedsTimer_i.Start();
			feedsTimer_i.Elapsed += new System.Timers.ElapsedEventHandler(feedsTimer_Elapsed);
		}
		#endregion ctor

		#region Properties


		/// <summary>
		/// This property holds the current ticker scroll speed
		/// </summary>
		public int CurrentScrollSpeed
		{
			get
			{
				return this.currentScrollSpeed_i;
			}
			set
			{
				this.currentScrollSpeed_i = value;
			}
		}

		/// <summary>
		/// This property holds the current Settings object
		/// </summary>
		public Settings Settings
		{
			get
			{
				return this.settings_i;
			}
		}

		/// <summary>
		/// This property returns the collection of bookmarks
		/// </summary>
		public Bookmarks Bookmarks
		{
			get
			{
				return bookmarks_i;
			}
			set
			{
				bookmarks_i = value;
			}
		}

		#endregion Properties

		#region Public Methods

		/// <summary>
		/// This method loads the bookmarks from the bookmarks.xml file
		/// If the file doesnt exist then it returns an empty Bookmarks object
		/// </summary>
		public Bookmarks LoadBookmarks()
		{
			try
			{
				return Bookmarks.Load("Bookmarks.xml");
			}
			catch
			{
				Bookmarks bookmarks = new Bookmarks();
				bookmarks.Save("Bookmarks.xml");
				return bookmarks;
			}
		}

		/// <summary>
		/// This method loads the bookmarks from the Bookmarkini.xm file
		/// If the file doesnt exist then it returns an empty Bookmarks object
		/// </summary>
		public Bookmarks LoadBookmarkIni()
		{
			try
			{
				return Bookmarks.Load("Bookmarkini.xml");
			}
			catch
			{
				Bookmarks bookmarks = new Bookmarks();
				bookmarks.Save("Bookmarkini.xml");
				return bookmarks;
			}
		}

		/// <summary>
		/// This method loads the settings from the settings.xml file
		/// If the file doesnt exist then it returns an empty settings object
		/// </summary>
		public Settings LoadSettings()
		{
			try
			{
				return Settings.Load("Settings.xml");
			}
			catch
			{
				Settings settings = new Settings();
				settings.Save("Settings.xml");
				return settings;
			}
		}

		/// <summary>
		/// This method creates an arraylist of labels containing the headlines for the latest versions of the feeds
		/// </summary>
		/// <returns></returns>
		public ArrayList GetNewSetOfLabels()
		{
			// Get the labels
			ArrayList labels = this.nextFeeds_i.GetLabels();
			
			// Set the location of each one
			foreach(LinkLabel label in labels)
			{
				label.Location = new Point(this.panelToUpdate_i.Width,1);
				label.Region = this.panelToUpdate_i.Region;
			}
			return labels;
		}

		/// <summary>
		/// This method returns a linklabel representing the next headline to be displayed
		/// </summary>
		/// <returns></returns>
		public LinkLabel GetNextLabel()
		{		
			LinkLabel returnLabel = (LinkLabel)this.currentSetOfLabels_i[currentLabelPos_i];
			return returnLabel;

		}

		/// <summary>
		/// This method increments the current headline position. If it is at 
		/// the end of a feed it moves on to the next one
		/// </summary>
		public void MoveToNextHeadlinePosition()
		{
			if(currentLabelPos_i == this.currentSetOfLabels_i.Count - 1)
			{	
				currentLabelPos_i = 0;
				this.currentSetOfLabels_i = this.GetNewSetOfLabels();
			}
			else
			{
				currentLabelPos_i++;
			}
		}

		/// <summary>
		/// This method returns a valid interval in milliseconds based on the inputted integer
		/// </summary>
		/// <param name="scrollSpeedSetting_p"></param>
		/// <returns></returns>
		public int GetScrollSpeedFromSetting(int scrollSpeedSetting_p)
		{
			return (100 - scrollSpeedSetting_p) + 15;
		}

		/// <summary>
		/// This method updates the feeds timer so that any changes to the setting's update period will take effect
		/// </summary>
		public void UpdateFeedsTimer()
		{
			// Initialise the feeds timer to update the feeds
			feedsTimer_i.Stop();
			feedsTimer_i.Interval = this.settings_i.UpdatePeriod * 1000;
			feedsTimer_i.Start();
		}

		/// <summary>
		/// This method updates all the feeds and sets the current set of labels to 
		/// reflect the new headlines. This method is used when the selected bookmarks has changed
		/// </summary>
		public void ForceFeedsUpdate()
		{
			// Update the feeds
			Feeds newFeeds_i = new Feeds(this.bookmarks_i);
			newFeeds_i.UpdateFeeds();
			newFeeds_i.Compare(this.nextFeeds_i);
			this.nextFeeds_i.FeedList.Clear();
			this.nextFeeds_i = newFeeds_i;
			
			// If the number of currently displayed labels is 0 then update the set of labels
			if(this.currentSetOfLabels_i.Count == 0)
			{
				this.currentSetOfLabels_i = this.GetNewSetOfLabels();
				restartTickThreadDelegate_i();
			}
		}

		#endregion Public Methods

		#region Event Handler Methods

		/// <summary>
		/// This method is called every time the timer period has elapsed
		/// </summary>
		public void feedsTimer_Elapsed(object source, System.Timers.ElapsedEventArgs args)
		{
			Feeds newFeeds_i = new Feeds(this.bookmarks_i);
			newFeeds_i.UpdateFeeds();
			newFeeds_i.Compare(this.nextFeeds_i);
			this.nextFeeds_i.FeedList.Clear();
			this.nextFeeds_i = newFeeds_i;
		}

		#endregion Event Handler Methods

	}
}
