/* -------------------------------------------------------------------------
 | Copyright (C) 2006  Ali Rawashdeh
 | This program is free software; you can redistribute it and/or modify
 | it under the terms of the GNU General Public License as published by
 | the Free Software Foundation; either version 2 of the License, or
 | (at your option) any later version.
 | 
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 | GNU General Public License for more details.
 | 
 | You should have received a copy of the GNU General Public License along
 | with this program (see License.txt); if not, write to the Free Software
 | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/
using System;
using System.Collections;
using System.Xml;
using System.Text;

namespace HomeDev.RSSTicker.Objects
{
	/// <summary>
	/// Summary description for Feed.
	/// </summary>
	public class Feed
	{
		// Declarations
		private string url_i;
		private int noOfHeadlines_i;
		private string name_i;
		private ArrayList headlines_i;

		#region ctor
		
		/// <summary>
		/// Instantiates a feed object using a bookmark's details
		/// </summary>
		/// <param name="bookmark_p"></param>
		public Feed(Bookmark bookmark_p)
		{
			this.url_i = bookmark_p.Url;
			this.noOfHeadlines_i = bookmark_p.NoOfHeadlines;
			this.name_i = bookmark_p.Name;
			this.headlines_i = new ArrayList();
		}

		#endregion ctor

		#region Properties

		/// <summary>
		/// This property holds the name of the RSS Feed Bookmark
		/// </summary>
		public string Name
		{
			get
			{
				return this.name_i;
			}
		}

		/// <summary>
		/// This property holds the URL of the RSS Feed
		/// </summary>
		public string Url
		{
			get
			{
				return this.url_i;
			}
		}

		/// <summary>
		/// This property holds the number of headlines to be used
		/// </summary>
		public int NoOfHeadlines
		{
			get
			{
				return this.noOfHeadlines_i;
			}
		}

		/// <summary>
		/// This property holds the headlines of the Feed
		/// </summary>
		public ArrayList Headlines
		{
			get
			{
				return this.headlines_i;
			}
		}

		#endregion Properties

		#region Public Methods

		/// <summary>
		/// This method updates the feed
		/// </summary>
		public bool UpdateFeed()
		{
			bool returnVal = true;
			this.headlines_i.Clear();

			try
			{
				// Fetch the feed and the root element
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.Load(this.url_i);
				XmlElement root = xmlDoc.DocumentElement;

				XmlNodeList itemList = root.GetElementsByTagName("item");

				// Go through each headline
				for(int i = 0; (i < itemList.Count) && (i < this.noOfHeadlines_i); i++)
				{
					XmlNode item = itemList[i];
					string title = String.Empty;
					string link = String.Empty;
					string description = String.Empty;
					string pubdate = String.Empty;

					// Go through each detail of the headline
					foreach(XmlNode detail in item.ChildNodes)
					{
						switch (detail.Name)
						{
							case "title":
							{
								title = detail.InnerText;
								break;
							}
							case "link":
							{
								link =  detail.InnerText;
								break;
							}
							case "description":
							{
								description =  detail.InnerText;
								break;
							}
							case "pubDate":
							{
								pubdate =  detail.InnerText;
								break;
							}
						}		
					}
					// Add the headline to the ArrayList
					this.headlines_i.Add(new Headline(title, link, description, pubdate));
				}
			}
			catch
			{
				returnVal = false;
			}
			return returnVal;
		}

		/// <summary>
		/// This method returns the contents of the Feed object
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			sb.Append("Feed: ");
			sb.Append(this.name_i);
			sb.Append(", " + this.url_i);
			sb.Append(", " + this.noOfHeadlines_i);
			foreach(Headline headline in this.headlines_i)
			{
				sb.Append("\n" + headline.ToString());
			}	
			return sb.ToString();
		}	
		#endregion Public Methods

	}
}
