/* -------------------------------------------------------------------------
 | Copyright (C) 2006  Ali Rawashdeh
 | This program is free software; you can redistribute it and/or modify
 | it under the terms of the GNU General Public License as published by
 | the Free Software Foundation; either version 2 of the License, or
 | (at your option) any later version.
 | 
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 | GNU General Public License for more details.
 | 
 | You should have received a copy of the GNU General Public License along
 | with this program (see License.txt); if not, write to the Free Software
 | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/
using System;
using System.Collections;
using System.Text;	
using System.Windows.Forms;
using System.Drawing;

namespace HomeDev.RSSTicker.Objects
{
	/// <summary>
	/// Summary description for Feeds.
	/// </summary>
	public class Feeds
	{
		// Declarations
		private Bookmarks bookmarks_i;
		private ArrayList feeds_i;

		#region ctor

		public Feeds(Bookmarks bookmarks_p)
		{
			this.bookmarks_i = bookmarks_p;
			this.feeds_i= new ArrayList();
		}

		#endregion ctor

		#region Properties

		/// <summary>
		/// This property holds an arraylist of Feed objects
		/// </summary>
		public ArrayList FeedList
		{
			get
			{
				return this.feeds_i;
			}
			set
			{
				this.feeds_i = value;
			}	
		}

		#endregion Properties

		#region Public Methods

		/// <summary>
		/// This method updates all the feeds based on the saved bookmarks
		/// </summary>
		public void UpdateFeeds()
		{
			// Go through each bookmark
			foreach(Bookmark bookmark in this.bookmarks_i.Items)
			{
				// Only get the feed if the bookmark's Display property is set to true
				if(bookmark.Display)
				{
					// Load the feed from the bookmark
					Feed feed = new Feed(bookmark);
					if(feed.UpdateFeed())
					{
						this.feeds_i.Add(feed);
					}
				}
			}
		}

		/// <summary>
		/// This method compares the current Feeds object with an old Feeds object.
		/// Any headlines which are new in the current Feeds object are set as new.
		/// </summary>
		/// <param name="feeds_p"></param>
		public void Compare(Feeds oldFeeds_p)
		{
			// Go through each new feed and headline
			foreach(Feed currentFeed in this.feeds_i)
			{
				foreach(Headline currentHeadline in currentFeed.Headlines)
				{	
					// Initialise the found variable as false
					bool found = false;
					// Go through each old feed and headline looking for a match
					foreach(Feed oldFeed in oldFeeds_p.feeds_i)
					{
						foreach(Headline oldHeadline in oldFeed.Headlines)
						{
							// If it matches then set found to true
							if((currentHeadline.Title == oldHeadline.Title)	&&
								(currentHeadline.Description == oldHeadline.Description))
							{
								found = true;
								break;
							}
						}
						if(found) break;
					}

					// If its found then IsNew is false, otherwise its true
					if(found)
					{
						currentHeadline.IsNew = false;
					}
					else
					{
						currentHeadline.IsNew = true;
					}
				}
			}
		}


		/// <summary>
		/// This method returns the contents of the Feeds object
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			sb.Append("Feeds: ");
			foreach(Feed feed in this.feeds_i)
			{
				sb.Append("\n" + feed.ToString());
			}	
			return sb.ToString();
		}	

		/// <summary>
		/// This method returns an arraylist of LinkLabel objects, one for each headline in each feed
		/// </summary>
		/// <returns></returns>
		public ArrayList GetLabels()
		{
			ArrayList returnList = new ArrayList();
			foreach(Feed feed in this.feeds_i)
			{
				LinkLabel headlabel = new LinkLabel();
				headlabel.Text = " --- " + feed.Name + " --- ";

				// Set the properties of the linklabel
				headlabel.LinkColor = System.Drawing.Color.White;
				headlabel.ActiveLinkColor = System.Drawing.Color.White;
				headlabel.Size = (new Size(50, 19));
				headlabel.Font = new System.Drawing.Font("Verdana", 9f, System.Drawing.FontStyle.Bold);
				headlabel.LinkBehavior = LinkBehavior.NeverUnderline;
				headlabel.AutoSize = true;
				headlabel.LinkArea = new LinkArea(0, headlabel.Text.Length);
				returnList.Add(headlabel);

				foreach(Headline headline in feed.Headlines)
				{
					LinkLabel label = new LinkLabel();
					// Set the text of the linklabel				
					label.Text = headline.Title;

					// Set the properties of the linklabel
					if(headline.IsNew)
					{
						label.LinkColor = System.Drawing.Color.PaleGoldenrod;
					}
					else
					{
						label.LinkColor = System.Drawing.Color.White;
					}
					label.ActiveLinkColor = System.Drawing.Color.Wheat;
					label.Links.Add(0,label.Text.Length, headline.Link);
					label.LinkClicked += new LinkLabelLinkClickedEventHandler(LabelLinkClick.LinkLabelClick);
					label.Size = (new Size(50, 19));
					label.Font = new System.Drawing.Font("Verdana", 9f, System.Drawing.FontStyle.Bold);
					label.LinkBehavior = LinkBehavior.NeverUnderline;
					label.AutoSize = true;

					returnList.Add(label);
				}
			}	
			return returnList;
		}

		#endregion Public Methods

	}
}
